@@ -11,13 +11,11 @@ use strict;
 use warnings;
 
 package Parse::ANSIColor::Tiny;
-# git description: v0.500-2-gaf3f174
-$Parse::ANSIColor::Tiny::VERSION = '0.501';
-BEGIN {
-  $Parse::ANSIColor::Tiny::AUTHORITY = 'cpan:RWSTAUNER';
-}
-# ABSTRACT: Determine attributes of ANSI-Colored string
+# git description: v0.501-13-g0dfbfe4
 
+our $AUTHORITY = 'cpan:RWSTAUNER';
+# ABSTRACT: Determine attributes of ANSI-Colored string
+$Parse::ANSIColor::Tiny::VERSION = '0.600';
 our @COLORS = qw( black red green yellow blue magenta cyan white );
 our %FOREGROUND = (
   (map { (               $COLORS[$_] =>  30 + $_ ) } 0 .. $#COLORS),
@@ -45,6 +43,44 @@ our %ATTRIBUTES = (
   %BACKGROUND,
 );
 
+# Generating the 256-color codes involves a lot of codes and offsets that are
+# not helped by turning them into constants.
+## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
+
+our @COLORS256;
+
+# The first 16 256-color codes are duplicates of the 16 ANSI colors,
+# included for completeness.
+for my $code (0 .. 15) {
+  my $name = "ansi$code";
+  $ATTRIBUTES{$name}      = "38;5;$code";
+  $ATTRIBUTES{"on_$name"} = "48;5;$code";
+  push @COLORS256, $name;
+}
+
+# 256-color RGB colors.  Red, green, and blue can each be values 0 through 5,
+# and the resulting 216 colors start with color 16.
+for my $r (0 .. 5) {
+  for my $g (0 .. 5) {
+    for my $b (0 .. 5) {
+      my $code = 16 + (6 * 6 * $r) + (6 * $g) + $b;
+      my $name = "rgb$r$g$b";
+      $ATTRIBUTES{$name}    = "38;5;$code";
+      $ATTRIBUTES{"on_$name"} = "48;5;$code";
+      push @COLORS256, $name;
+    }
+  }
+}
+
+# The last 256-color codes are 24 shades of grey.
+for my $n (0 .. 23) {
+  my $code = $n + 232;
+  my $name = "grey$n";
+  $ATTRIBUTES{$name}      = "38;5;$code";
+  $ATTRIBUTES{"on_$name"} = "48;5;$code";
+  push @COLORS256, $name;
+}
+
 # copied from Term::ANSIColor
   our %ATTRIBUTES_R;
   # Reverse lookup.  Alphabetically first name for a sequence is preferred.
@@ -72,28 +108,52 @@ sub new {
 
 
 sub colors {
-  return @COLORS;
+  return (@COLORS, @COLORS256);
 }
 sub foreground_colors {
-  return (@COLORS, map { "bright_$_" } @COLORS);
+  return (
+    @COLORS,
+    (map { "bright_$_" } @COLORS),
+    @COLORS256,
+  );
 }
 sub background_colors {
-  return ( (map { "on_$_" } @COLORS), (map { "on_bright_$_" } @COLORS) );
+  return (
+    (map { "on_$_" } @COLORS),
+    (map { "on_bright_$_" } @COLORS),
+    (map { "on_$_" } @COLORS256),
+  );
 }
 
 
+sub __separate_and_normalize {
+  my ($codes) = @_;
+
+  # Treat empty as "clear".
+  defined($codes) && length($codes)
+    or return 0;
+
+  # Replace empty (clear) with zero to simplify parsing and return values.
+  $codes =~ s/^;/0;/;
+  $codes =~ s/;$/;0/;
+  # Insert a zero between two semicolons (use look-ahead to get /g to find all).
+  $codes =~ s/;(?=;)/;0/g;
+
+  # Remove any leading zeros from (sections of) codes.
+  $codes =~ s/\b0+(?=\d)//g;
+
+  # Return all matches (of extended sequences or digits).
+  return $codes =~ m{ ( [34]8;5;\d+ | \d+) }xg;
+}
+
 sub identify {
   my ($self, @codes) = @_;
   local $_;
   return
     grep { defined }
-    map  { $ATTRIBUTES_R{ 0 + ($_ || 0) } }
-    map  { split /;/ }
-    # empty sequence is the same as clear so don't let split throw out empty ends
-    map  { m/;$/ ? $_ . '0' : $_ }
-    # prepending zero doesn't hurt and is probably better than s/^;/0;/
-    map  { '0' . ($_ || '0')  }
-    @codes;
+    map  { $ATTRIBUTES_R{ $_ } }
+    map  { __separate_and_normalize($_) }
+      @codes;
 }
 
 
@@ -271,7 +331,9 @@ Parse::ANSIColor::Tiny - Determine attributes of ANSI-Colored string
 
 =head1 VERSION
 
-version 0.501
+version 0.600
+
+=for test_synopsis sub h { shift };
 
 =head1 SYNOPSIS
 
@@ -360,13 +422,15 @@ Returns a list of the base color names (in numeric escape sequence order).
 
 Returns a list of the foreground colors (in numeric escape sequence order).
 
-This includes the base colors and the C<bright_> variants.
+This includes the base colors, their C<bright_> variants,
+and the names from the 256 palette (prefixes of C<ansi>, C<rgb>, and C<grey>).
 
 =head2 background_colors
 
 Returns a list of the background colors (in numeric escape sequence order).
 
-This includes the C<on_> and C<on_bright_> variants of the base colors.
+This includes the C<on_> and C<on_bright_> variants of the base colors
+and the C<on_> names for the 256 palette.
 
 =head2 identify
 
@@ -503,8 +567,6 @@ Function wrapped around L</parse>.
 
 Everything listed in L</FUNCTIONS> is also available for export upon request.
 
-=for test_synopsis sub h { shift };
-
 =head1 SEE ALSO
 
 =over 4
@@ -515,7 +577,7 @@ L<Term::ANSIColor> - For marking up text that will be printed to the terminal
 
 =item *
 
-L<Image::TextMode> (and L<Image::TextMode::Format::ANSI>) - Successor to L<Image::ANSI>; Specifically designed for parsing ANSI art
+L<Image::TextMode> (and L<Image::TextMode::Format::ANSI>) - Successor to C<Image::ANSI>; Specifically designed for parsing ANSI art
 
 =item *
 
@@ -555,7 +617,7 @@ L<http://metacpan.org/release/Parse-ANSIColor-Tiny>
 =head2 Bugs / Feature Requests
 
 Please report any bugs or feature requests by email to C<bug-parse-ansicolor-tiny at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Parse-ANSIColor-Tiny>. You will be automatically notified of any
+the web interface at L<https://rt.cpan.org/Public/Bug/Report.html?Queue=Parse-ANSIColor-Tiny>. You will be automatically notified of any
 progress on the request by the system.
 
 =head2 Source Code
@@ -569,6 +631,12 @@ L<https://github.com/rwstauner/Parse-ANSIColor-Tiny>
 
 Randy Stauner <rwstauner@cpan.org>
 
+=head1 CONTRIBUTOR
+
+=for stopwords Dmitry Fedin
+
+Dmitry Fedin <dmitry.fedin@gmail.com>
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2011 by Randy Stauner.
